import datetime
from datetime import timezone
import netCDF4
import numpy as np
import pyLARDA.helpers as h
import time
import git
import xarray as xr

# resolution of python version is master
class CalibratedSpectraXR(xr.Dataset):

    def __init__(self, _time, _range, *_vel):
        # build xarray dataset
        super().__init__()

        # set metadata
        if _time is not None:
            self.attrs['ts_unit'], self.attrs['ts_unit_long'] = 'sec', 'Unix time, seconds since Jan 1. 1979'
            self.attrs['dt_unit'], self.attrs['dt_unit_long'] = 'date', 'Datetime format'
            self.coords['ts'] = _time
            self.coords['dt'] = [h.ts_to_dt(ts) for ts in _time]
        if _range is not None:
            self.attrs['rg_unit'], self.attrs['rg_unit_long'] = 'm', 'Meter'
            self.coords['rg'] = _range

        # use cloudnet time and range resolution as default
        if len(_vel) > 0: self.coords['vel'] = _vel[0]
        if len(_vel) > 1: self.coords.update({f'vel_{ic + 1}': _vel[ic] for ic in range(1, len(_vel))})

    def _add_coordinate(self, name, unit, val):
        """
        Adding a coordinate to an xarray structure.

        Args:
            name (dict): key = variable name of the new coordinate, item = long name of the variable
            unit (string): variable unit
            val (numpy.array): values

        """
        for key, item in name.items():
            self.attrs[key] = item
            self.attrs[f'{key}_unit'] = unit
            self.coords[key] = val

    def add_nD_variable(self, name, dims, val, **kwargs):
        self[name] = (dims, val)
        for key, item in kwargs.items():
            self[name].attrs[key] = item


    
def write_simple_nc(fname, data_cont):
    """write a timeheight container into a simple netcdf file

    .. note::

        very early implementation, currently only working for
        ``dimlabel`` ``['time', 'range']``.

    Args:
        fname: filename
        data_cont: data_container
    """
    with netCDF4.Dataset(fname, mode='w',format='NETCDF4_CLASSIC') as ncfile:
        time_dim = ncfile.createDimension('time', None) # unlimited axis (can be appended to).
        range_dim = ncfile.createDimension('range', len(data_cont['rg']))
    
        unix = ncfile.createVariable('unix', np.float32, ('time',))
        unix.units = 'timestamp'
        unix.long_name = 'unix_timestamp'
        unix[:] = data_cont['ts']
        
        rg = ncfile.createVariable('range', np.float32, ('range',))
        rg.units = 'm'
        rg.long_name = 'range'
        rg[:] = data_cont['rg']
        
        var_str = f"{data_cont['system']}_{data_cont['name']}"
        v = ncfile.createVariable(
            var_str, np.float32, ('time', 'range'), fill_value=-999)
        v.units = data_cont['var_unit']
        v.var_lims = data_cont['var_lims']
        v.long_name = f"{data_cont['system']} {data_cont['name']}"
        
        var = data_cont['var'].copy()
        var[data_cont['mask']] = -999
        v[:] = var



def export_spectra2nc(data, larda_git_path='', system='', path='', **kwargs):
    """
    This routine generates an hourly NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        system (string): name of the radar system
        path (string): path where the NetCDF file is stored
    """


    hour_bias = kwargs['hour_bias'] if 'hour_bias' in kwargs else 0


    dt_start = h.ts_to_dt(data['time'][0])
    dt_end = h.ts_to_dt(data['time'][-1])
    ds_name = path + f'{dt_start:%Y%m%d-%H%M}-{dt_end:%H%M}_{system}_spectra.nc'

    repo = git.Repo(larda_git_path)
    sha = repo.head.object.hexsha

    print('open file: ', ds_name)

    with netCDF4.Dataset(ds_name, "w", format="NETCDF4") as ds:

        ds.git_description = f'GIT commit ID  {sha}'
        ds.description = f'{system} calibrated Doppler spectra'
        ds.history = 'Created ' + time.ctime(time.time())
        ds.system = system
        ds.location = data['location']
        ds._FillValue = data['fill_value']

        ds.createDimension('chirp', data['chirps'])  # add variable number of chirps later
        ds.createDimension('time', data['time'].size)
        ds.createDimension('range', data['range'].size)
        ds.createDimension('velocity', data['velocity'].shape[1])

        nc_add_variable(
            ds,
            val=data['coordinates'][0],
            dimension=(),
            var_name='latitude',
            type=np.float32,
            long_name='GPS latitude',
            unit='deg'
        )
        nc_add_variable(
            ds,
            val=data['coordinates'][1],
            dimension=(),
            var_name='longitude',
            type=np.float32,
            long_name='GPS longitude',
            unit='deg'
        )
        nc_add_variable(
            ds,
            val=data['altitude'],
            dimension=(),
            var_name='altitude',
            type=np.float32,
            long_name='altitude, i.e. height above sea level',
            unit='m'
        )
        nc_add_variable(
            ds,
            val=data['time'],
            dimension=('time',),
            var_name='time',
            type=np.float64,
            long_name='Unix Time - seconds since 01.01.1970 00:00 UTC',
            unit='sec'
        )
        nc_add_variable(
            ds,
            val=data['rg_offsets'],
            dimension=('chirp',),
            var_name='rg_offsets',
            type=np.float32,
            long_name='Range Indices for next chirp sequence.',
            unit='-'
        )
        nc_add_variable(
            ds,
            val=data['range'],
            dimension=('range',),
            var_name='range',
            type=np.float32,
            long_name='range',
            unit='m'
        )
        nc_add_variable(
            ds,
            val=data['velocity'],
            dimension=('chirp', 'velocity',),
            var_name=f'velocity_vectors',
            type=np.float32,
            long_name='velocity vectors for each chirp',
            unit='m s-1'
        )
        nc_add_variable(
            ds,
            val=data['nyquist_velocity'],
            dimension=('chirp',),
            var_name='nyquist_velocity',
            type=np.float32,
            long_name='Unambiguous Doppler velocity (+/-)',
            unit='m s-1'
        )
        nc_add_variable(
            ds,
            val=data['doppler_spectrum'],
            dimension=('time', 'range', 'velocity'),
            var_name='doppler_spectrum',
            type=np.float32,
            long_name='Doppler spectrum, if dual polarization radar: doppler_spectrum = vertical + horizontal polarization',
            unit='mm6 m-3 (m s-1)-1'
        )
        try:
            nc_add_variable(
                ds,
                val=data['covariance_spectrum_re'],
                dimension=('time', 'range', 'velocity'),
                var_name='covariance_spectrum_re',
                type=np.float32,
                long_name='Real part of covariance spectrum',
                unit='mm6 m-3'
            )
        except KeyError:
            print('skip writing real part of covariance spectrum')

            try:
                nc_add_variable(
                ds,
                val=data['covariance_spectrum_im'],
                dimension=('time', 'range', 'velocity'),
                var_name='covariance_spectrum_im',
                type=np.float32,
                long_name='Imaginary part of covariance spectrum',
                unit='mm6 m-3'
            )
            except KeyError:
                print('skip writing imaginary part of covariance spectrum')

        try:
            nc_add_variable(
                ds,
                val=data['sensitivity_limit'],
                dimension=('time', 'range'),
                var_name='sensitivity_limit',
                type=np.float32,
                long_name='Sensitivity limit, if dual polarization radar: sensitivity_limit = vertical + horizontal polarization',
                unit='mm6 m-3'
            )
        except KeyError:
            print('skip writing sensitivity limit')
        try:
            nc_add_variable(
                ds,
                val=data['doppler_spectrum_h'],
                dimension=('time', 'range', 'velocity'),
                var_name='doppler_spectrum_h',
                type=np.float32,
                long_name='Doppler spectrum, horizontal polarization only',
                unit='mm6 m-3   '
            )
            nc_add_variable(
                ds,
                val=data['sensitivity_limit_h'],
                dimension=('time', 'range'),
                var_name='sensitivity_limit_h',
                type=np.float32,
                long_name='Sensitivity limit for horizontal polarization',
                unit='mm6 m-3 (m s-1)-1'
            )
        except KeyError:
            print('Skip writing horizontal polarization.')

    return 0


def export_spectra_to_nc(data, system='', path='', **kwargs):
    """
    This routine generates an hourly NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        system (string): name of the radar system
        path (string): path where the NetCDF file is stored
    """

    no_chirps = len(data['rg_offsets']) - 1

    dt_start = h.ts_to_dt(data['ts'][0])
    dt_end = h.ts_to_dt(data['ts'][-1])
    ds_name = path + '{}-{}_{}_spectra.nc'.format(dt_start.strftime("%Y%m%d-%H%M"), dt_end.strftime("%H%M"), system)

    print('open file: ', ds_name)

    with netCDF4.Dataset(ds_name, "w", format="NETCDF4") as ds:

        # ds.commit_id = subprocess.check_output(["git", "describe", "--always"]).rstrip()
        ds.description = '{} calibrated Doppler spectra'.format(system)
        ds.history = 'Created ' + time.ctime(time.time())
        ds.system = system
        ds.location = data['paraminfo']['location']
        ds._FillValue = data['paraminfo']['fill_value']

        ds.createDimension('chirp', no_chirps)  # add variable number of chirps later
        ds.createDimension('time', data['ts'].size)
        for ic in range(no_chirps):
            ds.createDimension(f'C{ic + 1}range', data['rg'][ic].size)
            ds.createDimension(f'C{ic + 1}velocity', data['vel'][ic].size)

        nc_add_variable(ds, val=data['paraminfo']['coordinates'][0], dimension=(),
                        var_name='latitude', type=np.float32, long_name='GPS latitude', unit='deg')
        nc_add_variable(ds, val=data['paraminfo']['coordinates'][1], dimension=(),
                        var_name='longitude', type=np.float32, long_name='GPS longitude', unit='deg')
        nc_add_variable(ds, val=data['ts'], dimension=('time',),
                        var_name='time', type=np.float64, long_name='Unix Time - seconds since 01.01.1970 00:00 UTC', unit='sec')

        nc_add_variable(ds, val=data['rg_offsets'][:no_chirps], dimension=('chirp',),
                        var_name='rg_offsets', type=np.float32, long_name='Range Indices when Chirp shifts ', unit='-')

        for ic in range(no_chirps):
            nc_add_variable(ds, val=data['rg'][ic], dimension=(f'C{ic + 1}range',),
                            var_name=f'C{ic + 1}range', type=np.float32, long_name='range', unit='m')
            nc_add_variable(ds, val=data['vel'][ic], dimension=(f'C{ic + 1}velocity',),
                            var_name=f'C{ic + 1}vel', type=np.float32, long_name='velocity', unit='m s-1')
            nc_add_variable(ds, val=data['var'][ic], dimension=('time', f'C{ic + 1}range', f'C{ic + 1}velocity'),
                            var_name=f'C{ic + 1}Zspec', type=np.float32,
                            long_name=f'Doppler spectrum at vertical+horizontal polarization: Chirp {ic + 1}', unit='mm6 m-3')
            nc_add_variable(ds, val=data['vel'][ic][-1], dimension=('chirp',),
                            var_name=f'C{ic + 1}DoppMax', type=np.float32, long_name='Unambiguous Doppler velocity (+/-)', unit='m s-1')

    return 0


def rpg_radar2nc(data, path, larda_git_path, **kwargs):
    """
    This routine generates a daily NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        path (string): path where the NetCDF file is stored
    """

    dt_start = h.ts_to_dt(data['Ze']['ts'][0])

    h.make_dir(path)
    site_name = kwargs['site'] if 'site' in kwargs else 'no-site'
    cn_version = kwargs['version'] if 'version' in kwargs else 'pyhon'
    ds_name = f'{path}/{h.ts_to_dt(data["Ze"]["ts"][0]):%Y%m%d}-{site_name}-limrad94.nc'
    ncvers = '4'

    repo = git.Repo(larda_git_path)
    sha = repo.head.object.hexsha

    with netCDF4.Dataset(ds_name, 'w', format=f'NETCDF{ncvers}') as ds:
        ds.Convention = 'CF-1.0'
        ds.location = data['Ze']['paraminfo']['location']
        ds.system = data['Ze']['paraminfo']['system']
        ds.version = f'Variable names and dimensions prepared for Cloudnet {kwargs["version"]} version'
        ds.title = 'LIMRAD94 (SLDR) Doppler Cloud Radar, calibrated Input for Cloudnet'
        ds.institution = 'Leipzig Institute for Meteorology (LIM), Leipzig, Germany'
        ds.source = '94 GHz Cloud Radar LIMRAD94\nRadar type: Frequency Modulated Continuous Wave,\nTransmitter power 1.5 W typical (solid state ' \
                    'amplifier)\nAntenna Type: Bi-static Cassegrain with 500 mm aperture\nBeam width: 0.48deg FWHM'
        ds.reference = 'W Band Cloud Radar LIMRAD94\nDocumentation and User Manual provided by manufacturer RPG Radiometer Physics GmbH\n' \
                       'Information about system also available at https://www.radiometer-physics.de/'
        ds.calibrations = f'remove Precip. ghost: {kwargs["ghost_echo_1"]}\n, remove curtain ghost: {kwargs["ghost_echo_2"]}\n' \
                          f'despeckle: {kwargs["despeckle"]}\n, number of standard deviations above noise: {kwargs["NF"]}\n'

        ds.git_description = f'pyLARDA commit ID  {sha}'
        ds.description = 'Concatenated data files of LIMRAD 94GHz - FMCW Radar, used as input for Cloudnet processing, ' \
                         'filters applied: ghost-echo, despeckle, use only main peak'
        ds.history = 'Created ' + time.ctime(time.time())
        ds._FillValue = data['Ze']['paraminfo']['fill_value']

        ds.day = dt_start.day
        ds.month = dt_start.month
        ds.year = dt_start.year

        # ds.commit_id = subprocess.check_output(["git", "describe", "--always"]) .rstrip()
        ds.history = 'Created ' + time.ctime(time.time()) + '\nfilters applied: ghost-echo, despeckle, main peak only'

        Ze_str = 'Zh' if cn_version == 'python' else 'Ze'
        vel_str = 'v' if cn_version == 'python' else 'vm'
        width_str = 'width' if cn_version == 'python' else 'sigma'
        dim_tupel = ('time', 'range') if cn_version == 'python' else ('range', 'time')

        n_chirps = len(data['no_av'])
        ds.createDimension('chirp', n_chirps)
        ds.createDimension('time', data['Ze']['ts'].size)
        ds.createDimension('range', data['Ze']['rg'].size)

        if cn_version == 'matlab':
            for ivar in ['Ze', 'VEL', 'sw', 'ldr', 'kurt', 'skew']:
                data[ivar]['var'] = data[ivar]['var'].T

        # coordinates
        nc_add_variable(
            ds,
            val=94.0,
            dimension=(),
            var_name='frequency',
            type=np.float32,
            long_name='Radar frequency',
            units='GHz'
        )

        nc_add_variable(
            ds,
            val=256,
            dimension=(),
            var_name='Numfft',
            type=np.float32,
            long_name='Number of points in FFT',
            units=''
        )

        nc_add_variable(
            ds,
            val=np.mean(data['MaxVel']['var']),
            dimension=(),
            var_name='NyquistVelocity',
            type=np.float32,
            long_name='Mean (over all chirps) Unambiguous Doppler velocity (+/-)',
            units='m s-1'
        )

        nc_add_variable(
            ds,
            val=data['Ze']['paraminfo']['altitude'],
            dimension=(),
            var_name='altitude',
            type=np.float32,
            long_name='Height of instrument above mean sea level',
            units='m'
        )

        nc_add_variable(
            ds,
            val=data['Ze']['paraminfo']['coordinates'][0],
            dimension=(),
            var_name='latitude',
            type=np.float32,
            long_name='latitude',
            units='degrees_north'
        )

        nc_add_variable(
            ds,
            val=data['Ze']['paraminfo']['coordinates'][1],
            dimension=(),
            var_name='longitude',
            type=np.float32,
            long_name='longitude',
            units='degrees_east'
        )

        if 'version' in kwargs and cn_version == 'python':
            nc_add_variable(
                ds,
                val=data['no_av'],
                dimension=('chirp',),
                var_name='NumSpectraAveraged',
                type=np.float32,
                long_name='Number of spectral averages',
                units=''
            )

        # time and range variable
        # convert to time since midnight
        if cn_version == 'python':
            ts = np.subtract(data['Ze']['ts'], datetime.datetime(dt_start.year, dt_start.month, dt_start.day, 0, 0, 0, tzinfo=timezone.utc).timestamp()) / 3600
            ts_str = 'Decimal hours from midnight UTC to the middle of each day'
            ts_unit = f'hours since {dt_start:%Y-%m-%d} 00:00:00 +00:00 (UTC)'
            rg = data['Ze']['rg'] / 1000.0
        elif cn_version == 'matlab':
            ts = np.subtract(data['Ze']['ts'], datetime.datetime(2001, 1, 1, 0, 0, 0, tzinfo=timezone.utc).timestamp())
            ts_str = 'Seconds since 1st January 2001 00:00 UTC'
            ts_unit = 'sec'
            rg = data['Ze']['rg']
        else:
            raise ValueError('Wrong version selected! version to "matlab" or "python"!')

        nc_add_variable(ds, val=ts, dimension=('time',), var_name='time', type=np.float64, long_name=ts_str, units=ts_unit)
        nc_add_variable(ds, val=rg, dimension=('range',), var_name='range', type=np.float32,
                        long_name='Range from antenna to the centre of each range gate', units='km')
        nc_add_variable(ds, val=data['Azm']['var'], dimension=('time',), var_name='azimuth', type=np.float32,
                        long_name='Azimuth angle from north', units='degree')
        nc_add_variable(ds, val=data['Elv']['var'], dimension=('time',), var_name='elevation', type=np.float32,
                        long_name='elevation angle. 90 degree is vertical direction.', units='degree')

        # chirp dependent variables
        nc_add_variable(ds, val=data['MaxVel']['var'][0], dimension=('chirp',),
                        var_name='DoppMax', type=np.float32, long_name='Unambiguous Doppler velocity (+/-)', units='m s-1')

        # index plus (1 to n) for Matlab indexing
        nc_add_variable(ds, val=data['rg_offsets'], dimension=('chirp',),
                        var_name='range_offsets', type=np.int32,
                        long_name='chirp sequences start index array in altitude layer array', units='-')

        # 1D variables
        nc_add_variable(ds, val=data['bt']['var'], dimension=('time',),
                        var_name='bt', type=np.float32, long_name='Direct detection brightness temperature', units='K')

        nc_add_variable(ds, val=data['LWP']['var'], dimension=('time',),
                        var_name='lwp', type=np.float32, long_name='Liquid water path', units='g m-2')

        nc_add_variable(ds, val=data['rr']['var'], dimension=('time',),
                        var_name='rain', type=np.float32, long_name='Rain rate from weather station', units='mm h-1')

        nc_add_variable(ds, val=data['SurfRelHum']['var'], dimension=('time',),
                        var_name='SurfRelHum', type=np.float32, long_name='Relative humidity from weather station', units='%')

        # 2D variables
        nc_add_variable(ds, val=data['Ze']['var'], dimension=dim_tupel, var_name=Ze_str, type=np.float32,
                        long_name='Radar reflectivity factor', units='mm6 m-3', plot_range=data['Ze']['var_lims'], plot_scale='linear',
                        comment='Calibrated reflectivity. Calibration convention: in the absence of attenuation, '
                                'a cloud at 273 K containing one million 100-micron droplets per cubic metre will '
                                'have a reflectivity of 0 dBZ at all frequencies.')

        nc_add_variable(ds, val=data['VEL']['var'], dimension=dim_tupel, plot_range=data['VEL']['var_lims'], plot_scale='linear',
                        var_name=vel_str, type=np.float32, long_name='Doppler velocity', units='m s-1', unit_html='m s<sup>-1</sup>',
                        comment='This parameter is the radial component of the velocity, with positive velocities are away from the radar.',
                        folding_velocity=data['MaxVel']['var'].max())

        nc_add_variable(ds, val=data['sw']['var'], dimension=dim_tupel, plot_range=data['sw']['var_lims'], lot_scale='logarithmic',
                        var_name=width_str, type=np.float32, long_name='Spectral width', units='m s-1', unit_html='m s<sup>-1</sup>',
                        comment='This parameter is the standard deviation of the reflectivity-weighted velocities in the radar pulse volume.')

        nc_add_variable(ds, val=data['ldr']['var'], dimension=dim_tupel, plot_range=[-30.0, 0.0],
                        var_name='ldr', type=np.float32, long_name='Linear depolarisation ratio', units='dB',
                        comment='This parameter is the ratio of cross-polar to co-polar reflectivity.')

        nc_add_variable(ds, val=data['kurt']['var'], dimension=dim_tupel, plot_range=data['kurt']['var_lims'],
                        var_name='kurt', type=np.float32, long_name='Kurtosis', units='linear')

        nc_add_variable(ds, val=data['skew']['var'], dimension=dim_tupel, plot_range=data['skew']['var_lims'],
                        var_name='Skew', type=np.float32, long_name='Skewness', units='linear')

    print('save calibrated to :: ', ds_name)

    return 0


def rpg_radar2nc_eurec4a(data, path, **kwargs):
    """
    This routine generates a daily NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.
    The variables and metadata are designed for the EUREC4A campaign.

    Args:
        data (dict): dictionary of larda containers
        path (string): path where the NetCDF file is stored
    """

    dt_start = h.ts_to_dt(data['Ze']['ts'][0])

    h.make_dir(path)
    site_name = kwargs['site'] if 'site' in kwargs else 'no-site'
    cn_version = kwargs['version'] if 'version' in kwargs else 'python'
    hc_version = kwargs['heave_corr_version'] if 'heave_corr_version' in kwargs else None
    for_aeris = kwargs['for_aeris'] if 'for_aeris' in kwargs else False
    dataset_version = 'v1.2'
    ds_name = f'{path}/eurec4a_{site_name}_cloudradar_{h.ts_to_dt(data["Ze"]["ts"][0]):%Y%m%d}_{dataset_version}.nc'
    ncvers = '4'

    with netCDF4.Dataset(ds_name, 'w', format=f'NETCDF{ncvers}') as ds:
        ds.Conventions = 'CF-1.8'
        ds.title = 'LIMRAD94 (SLDR) Doppler Cloud Radar, calibrated file'
        ds.campaign_id = 'EUREC4A'
        ds.platform_id = 'Meteor'
        ds.instrument_id = 'LIMRAD94'
        ds.version_id = dataset_version
        ds.location = data['Ze']['paraminfo']['location']
        ds.system = data['Ze']['paraminfo']['system']
        ds.version = f'Variable names and dimensions prepared for upload to Aeris data center. 1.1: updated metadata; ' \
                     f'1.2: added hydrometeor_mask'
        ds.institution = 'Leipzig Institute for Meteorology (LIM), Leipzig, Germany'
        ds.contact = 'heike.kalesse@uni-leipzig.de'
        ds.source = '94 GHz Cloud Radar LIMRAD94\nRadar type: Frequency Modulated Continuous Wave,' \
                    '\nTransmitter power 1.5 W typical (solid state amplifier)' \
                    '\nAntenna Type: Bi-static Cassegrain with 500 mm aperture\nBeam width: 0.48deg FWHM'
        ds.reference = 'W Band Cloud Radar LIMRAD94' \
                       '\nDocumentation and User Manual provided by manufacturer RPG Radiometer Physics GmbH\n' \
                       'Information about system also available at https://www.radiometer-physics.de/'
        ds.calibrations = f'remove Precip. ghost: {kwargs["ghost_echo_1"]}\n, ' \
                          f'remove curtain ghost: {kwargs["ghost_echo_2"]}\n' \
                          f'despeckle: {kwargs["despeckle"]}\n, ' \
                          f'number of standard deviations above noise: {kwargs["NF"]}\n' \
                          f'spectra heave corrected: {kwargs["heave_correction"]}\n' \
                          f'spectra heave corrected with version: {hc_version}\n' \
                          f'spectra dealiased: {kwargs["dealiasing"]}'
        ds.description = 'Concatenated data files of LIMRAD 94GHz - FMCW Radar, ' \
                         'filters applied: ghost-echo, despeckle, use only main peak,\n' \
                         'spectra corrected for heave motion of ship and then dealiased'
        ds.history = 'Created ' + time.ctime(time.time())
        ds._FillValue = data['Ze']['paraminfo']['fill_value']
        ds.day = dt_start.day
        ds.month = dt_start.month
        ds.year = dt_start.year

        Ze_str = 'Zh' if cn_version == 'python' else 'Ze'
        vel_str = 'v' if cn_version == 'python' else 'vm'
        width_str = 'width' if cn_version == 'python' else 'sigma'
        dim_tupel = ('time', 'range') if cn_version == 'python' else ('range', 'time')

        n_chirps = len(data['no_av'])
        ds.createDimension('chirp', n_chirps)
        ds.createDimension('time', data['Ze']['ts'].size)
        ds.createDimension('range', data['Ze']['rg'].size)

        if cn_version == 'matlab':
            for ivar in ['Ze', 'VEL', 'sw', 'ldr', 'kurt', 'skew']:
                data[ivar]['var'] = data[ivar]['var'].T

        # scalar variables
        nc_add_variable(
            ds,
            val=94.0,
            dimension=(),
            var_name='frequency',
            type=np.float32,
            long_name='Radar frequency',
            units='GHz'
        )

        nc_add_variable(
            ds,
            val=256,
            dimension=(),
            var_name='Numfft',
            type=np.float32,
            long_name='Number of points in FFT',
            units='1'
        )
        if not for_aeris:
            nc_add_variable(
                ds,
                val=np.mean(data['MaxVel']['var']),
                dimension=(),
                var_name='NyquistVelocity',
                type=np.float32,
                long_name='Mean (over all chirps) Unambiguous Doppler velocity (+/-)',
                units='m s-1'
            )

        nc_add_variable(
            ds,
            val=data['Ze']['paraminfo']['altitude'],
            dimension=(),
            var_name='altitude',
            type=np.float32,
            long_name='Height of instrument above mean sea level',
            units='m'
        )

        if 'version' in kwargs and cn_version == 'python':
            nc_add_variable(
                ds,
                val=data['no_av'],
                dimension=('chirp',),
                var_name='NumSpectraAveraged',
                type=np.float32,
                long_name='Number of spectral averages',
                units='1'
            )

        # time and range variable
        # convert to time since start of year
        if cn_version == 'python':
            ts = np.subtract(data['Ze']['ts'], datetime.datetime(2020, 1, 1, 0, 0, 0, tzinfo=timezone.utc).timestamp())
            ts_str = 'seconds since 2020-01-01 00:00:00 UTC'
            ts_unit = f'seconds since 2020-01-01 00:00:00 +00:00 (UTC)'
            ts_calendar = 'standard'
            rg = data['Ze']['rg']
        elif cn_version == 'matlab':
            ts = np.subtract(data['Ze']['ts'], datetime.datetime(2001, 1, 1, 0, 0, 0, tzinfo=timezone.utc).timestamp())
            ts_str = 'Seconds since 1st January 2001 00:00 UTC'
            ts_unit = 'sec'
            rg = data['Ze']['rg']
        else:
            raise ValueError('Wrong version selected! Change version to "matlab" or "python"!')

        nc_add_variable(ds, val=ts, dimension=('time',), var_name='time', type=np.float64, long_name=ts_str,
                        units=ts_unit, calendar=ts_calendar)
        nc_add_variable(ds, val=rg, dimension=('range',), var_name='range', type=np.float32,
                        long_name='Range from antenna to the centre of each range gate', units='m')

        if for_aeris:
            # time dependent latitude, longitude variables
            nc_add_variable(ds, val=data['lat'], dimension=('time',), var_name='latitude', type=np.float32,
                            long_name='Latitude',
                            units='degrees_north')
            nc_add_variable(ds, val=data['lon'], dimension=('time',), var_name='longitude', type=np.float32,
                            long_name='Longitude',
                            units='degrees_east')
        else:
            # static lat lon variables
            nc_add_variable(
                ds,
                val=data['Ze']['paraminfo']['coordinates'][0],
                dimension=(),
                var_name='latitude',
                type=np.float32,
                long_name='latitude',
                units='degrees_north'
            )

            nc_add_variable(
                ds,
                val=data['Ze']['paraminfo']['coordinates'][1],
                dimension=(),
                var_name='longitude',
                type=np.float32,
                long_name='longitude',
                units='degrees_east'
            )

        # chirp dependent variables
        if for_aeris:
            nc_add_variable(ds, val=data['MaxVel']['var'][0], dimension=('chirp',),
                            var_name='Nyquist_velocity', type=np.float32, long_name='Unambiguous Doppler velocity (+/-)',
                            units='m s-1')
        else:
            nc_add_variable(ds, val=data['MaxVel']['var'][0], dimension=('chirp',),
                            var_name='DoppMax', type=np.float32,
                            long_name='Unambiguous Doppler velocity (+/-)',
                            units='m s-1')

        # index plus (1 to n) for Matlab indexing
        nc_add_variable(ds, val=data['rg_offsets'], dimension=('chirp',),
                        var_name='range_offsets', type=np.int32,
                        long_name='chirp sequences start index array in altitude layer array', units='1')

        if for_aeris:
            nc_add_variable(ds, val=data['time_shift_array'], dimension=('time', 'chirp'),
                            var_name='time_shift', type=np.float32,
                            long_name='Time shift between radar and ship calculated for each hour and chirp', units='s')

        # 1D variables
        dims_1d = ('time',)
        nc_add_variable(ds, val=data['bt']['var'], dimension=dims_1d,
                        var_name='bt', type=np.float32, long_name='Direct detection brightness temperature at 89 GHz',
                        units='K')

        nc_add_variable(ds, val=data['LWP']['var'], dimension=dims_1d,
                        var_name='lwp', type=np.float32, long_name='Liquid water path', units='g m-2')

        nc_add_variable(ds, val=data['rr']['var'], dimension=dims_1d,
                        var_name='rain', type=np.float32, long_name='Rain rate from weather station', units='mm h-1')

        nc_add_variable(ds, val=data['SurfRelHum']['var'], dimension=dims_1d,
                        var_name='SurfRelHum', type=np.float32, long_name='Relative humidity from weather station',
                        units='%')

        # 2D variables
        nc_add_variable(ds, val=data['Ze']['var'], dimension=dim_tupel, var_name=Ze_str, type=np.float32,
                        long_name='Radar reflectivity factor', units='dBZ', plot_range=data['Ze']['var_lims'],
                        plot_scale='linear',
                        comment='Calibrated reflectivity. Calibration convention: in the absence of attenuation, '
                                'a cloud at 273 K containing one million 100-micron droplets per cubic metre will '
                                'have a reflectivity of 0 dBZ at all frequencies.')

        nc_add_variable(ds, val=data['VEL_roll']['var'], dimension=dim_tupel, plot_range=data['VEL_roll']['var_lims'],
                        plot_scale='linear',
                        var_name=vel_str, type=np.float32, long_name='Best estimate of averaged mean Doppler velocity',
                        units='m s-1', unit_html='m s<sup>-1</sup>',
                        comment='This parameter is the radial component of the velocity, with positive velocities are '
                                'away from the radar (i.e. up) and negative velocities moving towards the radar '
                                '(i.e. down). It was corrected for the ships heave motion. '
                                'A rolling average over 3 time steps has been applied to it.',
                        folding_velocity=data['MaxVel']['var'].max())

        if for_aeris:
            nc_add_variable(ds, val=data['VEL']['var'], dimension=dim_tupel, plot_range=data['VEL']['var_lims'],
                            plot_scale='linear',
                            var_name=f'v_no_rolling_mean_applied', type=np.float32,
                            long_name='Heave corrected mean Doppler velocity', units='m s-1',
                            unit_html='m s<sup>-1</sup>',
                            comment='This parameter is the radial component of the velocity, with positive velocities '
                                    'are away from the radar (i.e. up) and negative velocities moving towards the '
                                    'radar (i.e. down). It was corrected for the ships heave motion but no rolling '
                                    'mean was applied.',
                            folding_velocity=data['MaxVel']['var'].max())

            nc_add_variable(ds, val=data['VEL_uncor']['var'], dimension=dim_tupel,
                            plot_range=data['VEL_uncor']['var_lims'], plot_scale='linear',
                            var_name='v_uncor', type=np.float32, long_name='Uncorrected Mean Doppler velocity',
                            units='m s-1', unit_html='m s<sup>-1</sup>',
                            comment='This parameter is the uncorrected radial component of the velocity, with positive '
                                    'velocities are away from the radar (i.e. up) and negative velocities moving '
                                    'towards the radar (i.e. down).',
                            folding_velocity=data['MaxVel']['var'].max())

        nc_add_variable(ds, val=data['sw']['var'], dimension=dim_tupel, plot_range=data['sw']['var_lims'],
                        plot_scale='logarithmic', var_name=width_str, type=np.float32, long_name='Spectral width',
                        units='m s-1', unit_html='m s<sup>-1</sup>',
                        comment='This parameter is the standard deviation of the reflectivity-weighted velocities '
                                'in the radar pulse volume.')

        nc_add_variable(ds, val=data['ldr']['var'], dimension=dim_tupel, plot_range=[-30.0, 0.0],
                        var_name='ldr', type=np.float32, long_name='Linear depolarisation ratio', units='dB',
                        comment='This parameter is the ratio of cross-polar to co-polar reflectivity.')

        nc_add_variable(ds, val=data['kurt']['var'], dimension=dim_tupel, plot_range=data['kurt']['var_lims'],
                        var_name='kurt', type=np.float32, long_name='Kurtosis', units='1')

        nc_add_variable(ds, val=data['skew']['var'], dimension=dim_tupel, plot_range=data['skew']['var_lims'],
                        var_name='Skew', type=np.float32, long_name='Skewness', units='1')

        if for_aeris:
            nc_add_variable(ds, val=data['heave_cor'], dimension=dim_tupel, plot_range=data['VEL']['var_lims'],
                            plot_scale='linear',
                            var_name='heave_cor', type=np.float32, long_name='Heave rate correction', units='m s-1',
                            unit_html='m s<sup>-1</sup>',
                            comment='This is the velocity by which the original Doppler spectrum was corrected by. '
                                    'The heave rate is subtracted from the Doppler velocity, meaning the spectra is '
                                    'shifted to the left for positive heave rates and to the right for negative heave '
                                    'rates.')

            nc_add_variable(ds, val=data['heave_cor_bins'], dimension=dim_tupel, plot_scale='linear',
                            var_name='heave_cor_bins', type=np.int32,
                            long_name='Heave rate correction in Doppler spectra bins', units='1',
                            comment='This is the number of bins by which the original Doppler spectrum was shifted by. '
                                    'Positive values shift the spectrum to the left, while negative values shift the '
                                    'spectrum to the right.')

            nc_add_variable(ds, val=data['cloud_bases_tops'], dimension=dim_tupel,
                            var_name='cloud_bases_tops', type=np.int32, long_name='Cloud Bases and Tops', units='1',
                            comment='Cloud Base: -1, Cloud Top: 1, Else: 0')

            nc_add_variable(ds, val=data['hydrometeor_mask'], dimension=dim_tupel,
                            var_name='hydrometeor_mask', type=np.int32, long_name='Hydrometeor Mask', units='1',
                            comment='Radar signal: 1, no signal: 0')

    print('save calibrated to :: ', ds_name)

    return 0


def rpg_radar2nc_old(data, path, **kwargs):
    """
    This routine generates a daily NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        path (string): path where the NetCDF file is stored
    """
    import time

    no_chirps = 3

    if 'time_frame' in kwargs:
        ds_name = path + kwargs['time_frame'] + '_LIMRAD94.nc'.format(h.ts_to_dt(data['Ze']['ts'][0]).strftime("%Y%m%d"),
                                                                      kwargs['time_frame'])
    else:
        ds_name = path + '{}_000000-240000_LIMRAD94.nc'.format(h.ts_to_dt(data['Ze']['ts'][0]).strftime("%Y%m%d"))

    with netCDF4.Dataset(ds_name, "w", format="NETCDF4") as ds:
        # ds.commit_id = subprocess.check_output(["git", "describe", "--always"]) .rstrip()
        ds.description = 'Concatenated data files of LIMRAD 94GHz - FMCW Radar, used as input for Cloudnet processing, ' \
                         'filters applied: ghos-echo, despeckle, use only main peak'
        ds.history = 'Created ' + time.ctime(time.time())
        ds.source = data['Ze']['paraminfo']['location']
        ds._FillValue = data['Ze']['paraminfo']['fill_value']

        ds.createDimension('chirp', no_chirps)  # add variable number of chirps later
        ds.createDimension('time', data['Ze']['ts'].size)
        ds.createDimension('range', data['Ze']['rg'].size)

        # coordinates
        nc_add_variable(ds, val=data['Ze']['paraminfo']['coordinates'][0], dimension=(),
                        var_name='latitude', type=np.float32, long_name='GPS latitude', unit='deg')

        nc_add_variable(ds, val=data['Ze']['paraminfo']['coordinates'][1], dimension=(),
                        var_name='longitude', type=np.float32, long_name='GPS longitude', unit='deg')

        # time and range variable
        # convert to time since 20010101
        ts = np.subtract(data['Ze']['ts'], datetime.datetime(2001, 1, 1, 0, 0, 0).timestamp())
        nc_add_variable(ds, val=ts, dimension=('time',),
                        var_name='time', type=np.float64, long_name='Seconds since 01.01.2001 00:00 UTC', unit='sec')

        nc_add_variable(ds, val=data['Ze']['rg'], dimension=('range',),
                        var_name='range', type=np.float32, long_name='range', unit='m')

        # 2D variables
        nc_add_variable(ds, val=data['Ze']['var'].T, dimension=('range', 'time',),
                        var_name='Ze', type=np.float32, long_name='Equivalent radar reflectivity factor', unit='mm^6/m^3')

        nc_add_variable(ds, val=data['VEL']['var'].T, dimension=('range', 'time',),
                        var_name='vm', type=np.float32, long_name='Mean Doppler velocity', unit='m/s')

        nc_add_variable(ds, val=data['sw']['var'].T, dimension=('range', 'time',),
                        var_name='sigma', type=np.float32, long_name='Spectrum width', unit='m/s')

        nc_add_variable(ds, val=data['ldr']['var'].T, dimension=('range', 'time',),
                        var_name='ldr', type=np.float32, long_name='Slanted linear depolarization ratio', unit='dB')

        nc_add_variable(ds, val=data['kurt']['var'].T, dimension=('range', 'time',),
                        var_name='kurt', type=np.float32, long_name='Kurtosis', unit='[linear]')

        nc_add_variable(ds, val=data['skew']['var'].T, dimension=('range', 'time',),
                        var_name='Skew', type=np.float32, long_name='Skewness', unit='[linear]')

        nc_add_variable(ds, val=data['DiffAtt']['var'].T, dimension=('range', 'time',),
                        var_name='DiffAtt', type=np.float32, long_name='Differential attenuation', unit='dB/km')

        # 1D variables
        nc_add_variable(ds, val=data['bt']['var'], dimension=('time',),
                        var_name='bt', type=np.float32, long_name='Direct detection brightness temperature', unit='K')

        nc_add_variable(ds, val=data['LWP']['var'], dimension=('time',),
                        var_name='lwp', type=np.float32, long_name='Liquid water path', unit='g/m^2')

        nc_add_variable(ds, val=data['rr']['var'], dimension=('time',),
                        var_name='rain', type=np.float32, long_name='Rain rate from weather station', unit='mm/h')

        nc_add_variable(ds, val=data['SurfRelHum']['var'], dimension=('time',),
                        var_name='SurfRelHum', type=np.float32, long_name='Relative humidity from weather station', unit='%')

        # chirp dependent variables
        nc_add_variable(ds, val=data['MaxVel']['var'][0], dimension=('chirp',),
                        var_name='DoppMax', type=np.float32, long_name='Unambiguous Doppler velocity (+/-)', unit='m/s')

        range_offsets = np.ones(no_chirps, dtype=np.int)
        for iC in range(no_chirps - 1):
            range_offsets[iC + 1] = range_offsets[iC] + data['C' + str(iC + 1) + 'Range']['var'][0].shape

        nc_add_variable(ds, val=range_offsets, dimension=('chirp',),
                        var_name='range_offsets', type=np.int,
                        long_name='chirp sequences start index array in altitude layer array', unit='[-]')

    print('save calibrated to :: ', ds_name)

    return 0


def nc_add_variable(nc_ds, **kwargs):
    """
    Helper function for adding a variable to a NetCDF file

    Args:
        nc_ds (NetCDF4 object): NetCDF data container with writing permission
        **var_name (string): variable name
        **type (numpy.int32, numpy.float32): variable type
        **dimension(tuple): dimensionality of the variable
        **val (numpy.array): values of the variable
        **long_name (string): more detailed description of the variable
        **unit (string): variable unit
    """
    try:
        _fillvalue = -999.0 if kwargs['type'] == np.float32 or kwargs['type'] == np.float64 else -2147483647
        var = nc_ds.createVariable(kwargs['var_name'], kwargs['type'], kwargs['dimension'], fill_value=_fillvalue)
        var[:] = kwargs['val']

        key_list = ['long_name', 'units', 'plot_range', 'folding_velocity', 'plot_scale', 'comment', 'unit_html',
                    'calendar']

        #        if len(kwargs['dimension']) > 0:
        #            kwargs['_FillValue'] = -999.0
        #            key_list.append('_FillValue')

        var.setncatts({f'{key}': kwargs[key] for key in key_list if key in kwargs})

    except Exception as e:
        raise e

    return var


def generate_weather_file_LIMRAD94(data, path, **kwargs):
    """
    This routine generates a daily NetCDF4 file with the weather station measurements from the RPG 94 GHz FMCW radar
    'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        path (string): path where the NetCDF file is stored
    """
    import time

    if 'time_frame' in kwargs:
        ds_name = path + kwargs['time_frame'] + \
                  '_LIMRAD94_weather.nc'.format(h.ts_to_dt(data['SurfWS']['ts'][0]).strftime("%Y%m%d"), kwargs['time_frame'])
    else:
        ds_name = path + '{}_000000-240000_LIMRAD94_weather.nc'.format(h.ts_to_dt(data['SurfWS']['ts'][0]).strftime("%Y%m%d"))

    ds = netCDF4.Dataset(ds_name, "w", format="NETCDF4")

    ds.description = 'Concatenated data files of LIMRAD 94GHz - FMCW Radar, Data from the Vaisalla weather station'
    ds.history = 'Created ' + time.ctime(time.time())
    ds.source = data['SurfWS']['paraminfo']['location']
    ds.FillValue = data['SurfWS']['paraminfo']['fill_value']

    ds.createDimension('time', data['SurfWS']['ts'].size)

    # coordinates
    nc_add_variable(ds, val=data['SurfWS']['paraminfo']['coordinates'][0], dimension=(),
                    var_name='latitude', type=np.float32, long_name='GPS latitude', unit='deg')

    nc_add_variable(ds, val=data['SurfWS']['paraminfo']['coordinates'][1], dimension=(),
                    var_name='longitude', type=np.float32, long_name='GPS longitude', unit='deg')

    # time
    # convert to time since 20010101
    ts = np.subtract(data['SurfWS']['ts'], datetime.datetime(2001, 1, 1, 0, 0, 0).timestamp())
    nc_add_variable(ds, val=ts, dimension=('time',),
                    var_name='time', type=np.float64, long_name='Seconds since 2001-01-01 00:00 UTC', unit='seconds')

    # 1D variables
    nc_add_variable(ds, val=data['SurfRelHum']['var'], dimension=('time',),
                    var_name='SurfRelHum', type=np.float32,
                    long_name='Relative humidity from weather station', unit='%')

    nc_add_variable(ds, val=data['SurfTemp']['var'], dimension=('time',),
                    var_name='SurfTemp', type=np.float32, long_name='Surface temperature from weather station',
                    unit='K')

    nc_add_variable(ds, val=data['SurfPres']['var'], dimension=('time',),
                    var_name='SurfPres', type=np.float32,
                    long_name='Surface atmospheric pressure from weather station', unit='hPa')

    nc_add_variable(ds, val=data['SurfWS']['var'], dimension=('time',),
                    var_name='SurfWS', type=np.float32, long_name='Surface wind speed from weather station',
                    unit='m/s')

    nc_add_variable(ds, val=data['SurfWD']['var'], dimension=('time',),
                    var_name='SurfWD', type=np.float32,
                    long_name='Surface wind direction from weather station',
                    unit='deg')

    nc_add_variable(ds, val=data['rr']['var'], dimension=('time',),
                    var_name='Rain', type=np.float32, long_name='Rain rate from weather station', unit='mm/h')

    ds.close()

    print('save calibrated to :: ', ds_name)

    return 0


def generate_30s_averaged_Ze_files(data, path, **kwargs):
    """
    This routine generates a daily NetCDF4 file for the RPG 94 GHz FMCW radar 'LIMRAD94'.

    Args:
        data (dict): dictionary of larda containers
        path (string): path where the NetCDF file is stored
    """
    import time
    ds_name = path + f"RV-METEOR_LIMRAD94_Ze_{h.ts_to_dt(data['Ze']['ts'][0]):%Y%m%d}.nc"
    ds = netCDF4.Dataset(ds_name, "w", format="NETCDF4")
    # ds.commit_id = subprocess.check_output(["git", "describe", "--always"]) .rstrip()
    ds.description = 'Preliminary LIMRAD 94GHz - FMCW Doppler radar data, averaged to 30s and 30m time/range resolution \n' \
                     'filters applied: ghost-echo; despeckle; the reflectivity calculated is only based on the main ' \
                     'radar peak if several are separated by the noise floor \n ' \
                     'Institution: Leipzig University - Institute for Meteorology \n ' \
                     'Contact: heike.kalesse@uni-leipzig.de'
    ds.history = 'Created ' + time.ctime(time.time()) + ' UTC'
    ds.source = data['Ze']['paraminfo']['location']
    ds.FillValue = data['Ze']['paraminfo']['fill_value']
    ds.createDimension('time', data['Ze']['ts'].size)
    ds.createDimension('range', data['Ze']['rg'].size)

    # coordinates
    nc_add_variable(ds, val=data['Ze']['paraminfo']['coordinates'][0], dimension=(),
                    var_name='latitude', type=np.float32, long_name='GPS latitude', unit='deg')

    nc_add_variable(ds, val=data['Ze']['paraminfo']['coordinates'][1], dimension=(),
                    var_name='longitude', type=np.float32, long_name='GPS longitude', unit='deg')

    # time and range variable
    # convert to time since 20010101
    ts = np.subtract(data['Ze']['ts'], datetime.datetime(2001, 1, 1, 0, 0, 0).timestamp())
    nc_add_variable(ds, val=ts, dimension=('time',),
                    var_name='time', type=np.float64, long_name='Seconds since 2001-01-01 00:00 UTC', unit='seconds')

    nc_add_variable(ds, val=data['Ze']['rg'], dimension=('range',),
                    var_name='range', type=np.float32, long_name='range', unit='m')

    # 2D variables
    nc_add_variable(ds, val=data['Ze']['var'], dimension=('time', 'range',),
                    var_name='Ze', type=np.float32, long_name='Linearly averaged equivalent radar reflectivity factor',
                    unit='mm^6/m^3')
    nc_add_variable(ds, val=data['Ze']['cloud_mask'], dimension=('time', 'range',),
                    var_name='cloud_bases_tops', type=np.int16,
                    long_name='Hydrometeor layer bases (-1) and hydrometeor layer tops (1)',
                    unit='[-]')

    ds.close()
    print('save calibrated to :: ', ds_name)

    return 0



#!/usr/bin/python3

import sys
# just needed to find pyLARDA from this location
sys.path.append('../')
sys.path.append('.')


import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

import pyLARDA
import pyLARDA.helpers as h
import datetime
import numpy as np
import scipy.ndimage as spn
from scipy import stats

#Load LARDA
larda = pyLARDA.LARDA().connect('lacros_dacapo')
c_info = [larda.camp.LOCATION, larda.camp.VALID_DATES]

print(larda.days_with_data())
#print("array_avail()", larda.array_avail(2015, 6))
#print("single month with new interface ", larda.instr_status(2015, 6)) 


#begin_dt=datetime.datetime(2018,12,6,0,1)
begin_dt=datetime.datetime(2018,12,6,1,40)
end_dt=datetime.datetime(2018,12,6,4,0,0)

plot_range = [300, 10000]
case_prefix = 'plots/scatter_case_studies/comparison_cloud_'

#### load the velocity data

MIRA_VELg=larda.read("MIRA","VELg",[begin_dt,end_dt],[0,'max'])
MIRA_VELg['var_lims'] = [-6,6]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_VELg, range_interval=plot_range)
fig.savefig(case_prefix+'mira_vel.png', dpi=250)

LIMRAD94_VEL=larda.read("LIMRAD94","VEL",[begin_dt,end_dt],[0,'max'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel.png', dpi=250)
LIMRAD94_VEL_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_VEL, new_time=MIRA_VELg['ts'], new_range=MIRA_VELg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL_interp, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel_interp.png', dpi=250)

# vel scatterplot
combined_mask = np.logical_or(MIRA_VELg['mask'], LIMRAD94_VEL_interp['mask'])

Mira_VELg_scatter = MIRA_VELg['var'][~combined_mask].ravel()
Limrad_VEL_scatter = LIMRAD94_VEL_interp['var'][~combined_mask].ravel()


s, i, r, p, std_err = stats.linregress(Mira_VELg_scatter, Limrad_VEL_scatter)
H, xedges, yedges = np.histogram2d(Mira_VELg_scatter, Limrad_VEL_scatter, 
                                   bins=120, range=[[-4, 4], [-4, 4]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-10,10],[-10,10], color='salmon')
ax.plot([-10,10],[-8,12], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-9,11], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-11,9], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-12,8], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-4,4])
ax.set_ylim([-4,4])
ax.set_xlabel("MIRA VELg")
ax.set_ylabel("LIMRAD94 VEL")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_vel_mira_limrad.png', dpi=250)



# other day for the mrr
begin_dt=datetime.datetime(2018,12,9,2,0)
end_dt=datetime.datetime(2018,12,9,5,0,0)
case_prefix = 'plots/scatter_case_studies/comparison_rain_'

MIRA_VELg=larda.read("MIRA","VELg",[begin_dt,end_dt],[0,4000])
MIRA_VELg['var_lims'] = [-6,6]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_VELg, range_interval=plot_range)
fig.savefig(case_prefix+'mira_vel.png', dpi=250)

LIMRAD94_VEL=larda.read("LIMRAD94","VEL",[begin_dt,end_dt],[0,4000])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel.png', dpi=250)
LIMRAD94_VEL_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_VEL, new_time=MIRA_VELg['ts'], new_range=MIRA_VELg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL_interp, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel_interp.png', dpi=250)

# vel scatterplot
combined_mask = np.logical_or(MIRA_VELg['mask'], LIMRAD94_VEL_interp['mask'])

Mira_VELg_scatter = MIRA_VELg['var'][~combined_mask].ravel()
Limrad_VEL_scatter = LIMRAD94_VEL_interp['var'][~combined_mask].ravel()

s, i, r, p, std_err = stats.linregress(Mira_VELg_scatter, Limrad_VEL_scatter)
H, xedges, yedges = np.histogram2d(Mira_VELg_scatter, Limrad_VEL_scatter, 
                                   bins=120, range=[[-4, 4], [-4, 4]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-10,10],[-10,10], color='salmon')
ax.plot([-10,10],[-8,12], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-9,11], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-11,9], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-12,8], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-4,4])
ax.set_ylim([-4,4])
ax.set_xlabel("MIRA VELg")
ax.set_ylabel("LIMRAD94 VEL")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_vel_mira_limrad.png', dpi=250)


MRR_VEL=larda.read("MRRPRO","VEL",[begin_dt,end_dt],[0,4000])
fig, ax = pyLARDA.Transformations.plot2d(MRR_VEL, range_interval=plot_range)
fig.savefig(case_prefix+'mrr_vel.png', dpi=250)
MRR_VEL_interp = pyLARDA.Transformations.interpolate2d(MRR_VEL, new_time=MIRA_VELg['ts'], new_range=MIRA_VELg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(MRR_VEL_interp, range_interval=plot_range)
fig.savefig(case_prefix+'mrr_vel_interp.png', dpi=250)

# scatterplot
combined_mask = np.logical_or(MIRA_VELg['mask'], MRR_VEL_interp['mask'])

Mira_VELg_scatter = MIRA_VELg['var'][~combined_mask].ravel()
MRR_VEL_scatter = MRR_VEL_interp['var'][~combined_mask].ravel()

s, i, r, p, std_err = stats.linregress(Mira_VELg_scatter, MRR_VEL_scatter)
H, xedges, yedges = np.histogram2d(Mira_VELg_scatter, MRR_VEL_scatter, 
                                   bins=120, range=[[-4, 4], [-4, 4]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-10,10],[-10,10], color='salmon')
ax.plot([-10,10],[-8,12], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-9,11], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-11,9], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-12,8], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-4,4])
ax.set_ylim([-4,4])
ax.set_xlabel("MIRA VELg")
ax.set_ylabel("MRRPRO VEL")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_vel_mira_mrr.png', dpi=250)


MRR_VEL_interp = pyLARDA.Transformations.interpolate2d(MRR_VEL, new_time=LIMRAD94_VEL['ts'], new_range=LIMRAD94_VEL['rg'])
fig, ax = pyLARDA.Transformations.plot2d(MRR_VEL_interp, range_interval=plot_range)
fig.savefig(case_prefix+'mrr_vel_interp_to_limrad.png', dpi=250)

combined_mask = np.logical_or(LIMRAD94_VEL['mask'], MRR_VEL_interp['mask'])

LIMRAD94_VEL_scatter = LIMRAD94_VEL['var'][~combined_mask].ravel()
MRR_VEL_scatter = MRR_VEL_interp['var'][~combined_mask].ravel()

s, i, r, p, std_err = stats.linregress(LIMRAD94_VEL_scatter, MRR_VEL_scatter)
H, xedges, yedges = np.histogram2d(LIMRAD94_VEL_scatter, MRR_VEL_scatter, 
                                   bins=120, range=[[-4, 4], [-4, 4]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-10,10],[-10,10], color='salmon')
ax.plot([-10,10],[-8,12], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-9,11], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-11,9], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-12,8], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-4,4])
ax.set_ylim([-4,4])
ax.set_xlabel("LIMRAD VEL")
ax.set_ylabel("MRRPRO VEL")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_vel_limrad_mrr.png', dpi=250)



begin_dt=datetime.datetime(2018,12,14,6,0)
end_dt=datetime.datetime(2018,12,14,9,0)

plot_range = [300, 10000]
case_prefix = 'plots/scatter_case_studies/comparison_c14_'

#### load the velocity data

MIRA_VELg=larda.read("MIRA","VELg",[begin_dt,end_dt],[0,'max'])
MIRA_VELg['var_lims'] = [-6,6]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_VELg, range_interval=plot_range)
fig.savefig(case_prefix+'mira_vel.png', dpi=250)

LIMRAD94_VEL=larda.read("LIMRAD94","VEL",[begin_dt,end_dt],[0,'max'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel.png', dpi=250)
LIMRAD94_VEL_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_VEL, new_time=MIRA_VELg['ts'], new_range=MIRA_VELg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_VEL_interp, range_interval=plot_range)
fig.savefig(case_prefix+'limrad_vel_interp.png', dpi=250)

# vel scatterplot
combined_mask = np.logical_or(MIRA_VELg['mask'], LIMRAD94_VEL_interp['mask'])

Mira_VELg_scatter = MIRA_VELg['var'][~combined_mask].ravel()
Limrad_VEL_scatter = LIMRAD94_VEL_interp['var'][~combined_mask].ravel()


s, i, r, p, std_err = stats.linregress(Mira_VELg_scatter, Limrad_VEL_scatter)
H, xedges, yedges = np.histogram2d(Mira_VELg_scatter, Limrad_VEL_scatter, 
                                   bins=120, range=[[-4, 4], [-4, 4]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-10,10],[-10,10], color='salmon')
ax.plot([-10,10],[-8,12], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-9,11], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-11,9], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-10,10],[-12,8], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-4,4])
ax.set_ylim([-4,4])
ax.set_xlabel("MIRA VELg")
ax.set_ylabel("LIMRAD94 VEL")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_vel_mira_limrad.png', dpi=250)



#!/usr/bin/python3

import sys

# just needed to find pyLARDA from this location
sys.path.append('../')
sys.path.append('.')

import matplotlib

matplotlib.use('Agg')
import pyLARDA
import datetime
import numpy as np

import logging

log = logging.getLogger('pyLARDA')
log.setLevel(logging.INFO)
log.addHandler(logging.StreamHandler())

# Load LARDA
larda = pyLARDA.LARDA().connect('lacros_dacapo')
c_info = [larda.camp.LOCATION, larda.camp.VALID_DATES]

print('available systems:', larda.connectors.keys())
print("available parameters: ", [(k, larda.connectors[k].params_list) for k in larda.connectors.keys()])
print('days with data', larda.days_with_data())

begin_dt = datetime.datetime(2019, 2, 11, 13, 0, 0)
end_dt = datetime.datetime(2019, 2, 11, 15, 59, 59)
plot_range = [0, 1500]

"""
    Create frequency of occurrence plot for reflectivity values
"""
# load first LIMRAD dict to gather some more information
LIMRAD94_Ze = larda.read("LIMRAD94", "Ze", [begin_dt, end_dt], plot_range)

# load range_offsets, dashed lines where chirp shifts
range_C1 = larda.read("LIMRAD94", "C1Range", [begin_dt, end_dt], plot_range)['var'].max()
range_C2 = larda.read("LIMRAD94", "C2Range", [begin_dt, end_dt], plot_range)['var'].max()

# load sensitivity limits (time, height) and calculate the mean over time
LIMRAD94_SLv = larda.read("LIMRAD94", "SLv", [begin_dt, end_dt], plot_range)
sens_lim = np.mean(LIMRAD94_SLv['var'], axis=0)

# create frequency of occurrence plot of LIMRAD94 reflectivity and save as png
titlestring = 'LIMRAD94 Ze -- date: {}'.format(begin_dt.strftime("%Y-%m-%d"))
fig, ax = pyLARDA.Transformations.plot_frequency_of_occurrence(LIMRAD94_Ze, x_lim=[-70, 10], y_lim=plot_range,
                                                               sensitivity_limit=sens_lim, z_converter='lin2z',
                                                               range_offset=[range_C1, range_C2], title=titlestring)

fig.savefig('limrad_FOC.png', dpi=250)





cimport cython
import numpy as np


@cython.boundscheck(False)  # Deactivate bounds checking
@cython.wraparound(False)   # Deactivate negative indexing.
def build_tree(double[:, :] data, bint ldr_avail):
    """
    'parent' #0
    'Z' #1
    'v' #2
    'width' #3
    'skew' #4
    'threshold' #5
    'prominence' #6
    'bound_l' #7
    'bound_r' #8
    'LDR' #9
    'ldrmax' #10
    """
    cdef dict travtree = {}    
    cdef dict node
    cdef int k
    
    #logger.debug('peakTree parent {}'.format(ncD.variables['parent'][it,ir,:]))
    avail_nodes = np.argwhere(np.asarray(data[:,0]) != -999.).ravel()
    #print("within build_tree", avail_nodes)
    for k in np.asarray(avail_nodes).astype(int).tolist():
        node = {'parent_id': data[k,0], 
                'thres': data[k,5],
                'width': data[k,3], 
                'z': data[k,1], 
                'bounds': (data[k,7], data[k,8]),
                'skew': data[k,4],
                'prominence': data[k,6],
                'v': data[k,2]}
        #print("done build node, now configuring")
        node['id'] = k
        node['bounds'] = list(map(int, node['bounds']))
        node['width'] = node['width'] if np.isfinite(node['width']) else -99
        node['skew'] = node['skew'] if np.isfinite(node['skew']) else -99
        node['thres'] = node['thres'] if np.isfinite(node['thres']) else -99
        node['prominence'] = node['prominence'] if np.isfinite(node['prominence']) else -99
        if ldr_avail:
            node['ldr'] = data[k,9]
            node['ldr'] = node['ldr'] if np.isfinite(node['ldr']) else -99
            node['ldrmax'] = data[k,10]
            node['ldrmax'] = node['ldrmax'] if np.isfinite(node['ldrmax']) else -99
        else:
            node['ldr'], node['ldrmax'] = -99, -99
        #print("done build node, now configuring")
        if node['parent_id'] != -999:
            if k == 0:
                node['coords'] = [0]
            else:
                coords = travtree[node['parent_id']]['coords']
                if k%2 == 0:
                    node['coords'] = coords + [1]
                else:
                    node['coords'] = coords + [0]
            
            # remove the parent id for compatibility to peakTreeVis
            if node['parent_id'] == -1:
                del node['parent_id']
            # format for transport
            #v = {ky: format_for_json(val) for ky, val in v.items()}
            travtree[k] = node
            
    return travtree

@cython.boundscheck(False)  # Deactivate bounds checking
@cython.wraparound(False)   # Deactivate negative indexing.
def array_to_tree_c(double[:, :, :, :] data, bint ldr_avail):
    trees = np.empty((data.shape[0], data.shape[1]), dtype=object)
    mask = np.empty((data.shape[0], data.shape[1]), dtype=bool)
    mask[:] = True
    cdef double[:,:,:,:] data_view = data
    cdef int it, ir
    
    for it in range(data.shape[0]):
        #print(it)
        for ir in range(data.shape[1]):
            #print(np.asarray(data[it,ir,:,0]))
            trees[it, ir] = build_tree(data[it,ir,:,:], ldr_avail)
            mask[it, ir] = False
            
    return trees, mask


# Copyright (c) 2013-2015 University Corporation for Atmospheric Research/Unidata.
# Distributed under the terms of the MIT License.
# SPDX-License-Identifier: MIT
"""Read upper air data from the Wyoming archives."""

#!/usr/bin/python3

from io import StringIO
import warnings
from bs4 import BeautifulSoup
import numpy as np
import pandas as pd
from siphon._tools import get_wind_components
from siphon.http_util import HTTPEndPoint
import datetime
import metpy.units as units
import pyLARDA.helpers as h

warnings.filterwarnings('ignore', 'Pandas doesn\'t allow columns to be created', UserWarning)

class WyomingUpperAir(HTTPEndPoint):
    """Download and parse data from the University of Wyoming's upper air archive."""

    def __init__(self):
        """Set up endpoint."""
        super(WyomingUpperAir, self).__init__('http://weather.uwyo.edu/cgi-bin/sounding')

    @classmethod
    def request_data(cls, time, site_id, **kwargs):
        r"""Retrieve upper air observations from the Wyoming archive.

        Parameters
        ----------
        time : datetime
            The date and time of the desired observation.

        site_id : str
            The three letter ICAO identifier of the station for which data should be
            downloaded.

        kwargs
            Arbitrary keyword arguments to use to initialize source

        Returns
        -------
            :class:`pandas.DataFrame` containing the data

        """
        endpoint = cls()
        df = endpoint._get_data(time, site_id, **kwargs)
        return df

    def _get_data(self, time, site_id, region='naconf'):
        r"""Download and parse upper air observations from an online archive.

        Parameters
        ----------
        time : datetime
            The date and time of the desired observation.

        site_id : str
            The three letter ICAO identifier of the station for which data should be
            downloaded.

        region
            Region to request data from

        Returns
        -------
            :class:`pandas.DataFrame` containing the data

        """
        raw_data, meta_data = self._get_data_raw(time, site_id, region)
        col_names = ['pressure', 'range', 'temperature', 'dewpoint', 'direction', 'speed']
        df = pd.read_fwf(raw_data, skiprows=5, usecols=[0, 1, 2, 3, 6, 7], names=col_names)
        df['u_wind'], df['v_wind'] = get_wind_components(df['speed'],
                                                         np.deg2rad(df['direction']))

        # Drop any rows with all NaN values for T, Td, winds
        df = df.dropna(subset=('temperature', 'dewpoint', 'direction', 'speed',
                               'u_wind', 'v_wind'), how='all').reset_index(drop=True)

        # Add unit dictionary
        df.units = {'pressure': 'hPa',
                    'range': 'meter',
                    'temperature': 'degC',
                    'dewpoint': 'degC',
                    'direction': 'degrees',
                    'speed': 'knot',
                    'u_wind': 'knot',
                    'v_wind': 'knot'}
        for item in list(meta_data.split('\n'))[1:-1]:
            var, value = item.split(': ')
            df._metadata.append({var.strip(): value})
        return df

    def _get_data_raw(self, time, site_id, region='naconf'):
        """Download data from the University of Wyoming's upper air archive.

        Parameters
        ----------
        time : datetime
            Date and time for which data should be downloaded
        site_id : str
            Site id for which data should be downloaded
        region : str
            The region in which the station resides. Defaults to `naconf`.

        Returns
        -------
        a file-like object from which to read the data

        """
        path = ('?region={region}&TYPE=TEXT%3ALIST'
                '&YEAR={time:%Y}&MONTH={time:%m}&FROM={time:%d%H}&TO={time:%d%H}'
                '&STNM={stid}').format(region=region, time=time, stid=site_id)

        resp = self.get_path(path)
        # See if the return is valid, but has no data
        if resp.text.find('Can\'t') != -1:
            raise ValueError(
                'No data available for {time:%Y-%m-%d %HZ} from region {region} '
                'for station {stid}.'.format(time=time, region=region,
                                             stid=site_id))

        soup = BeautifulSoup(resp.text, 'html.parser')
        return StringIO(soup.find_all('pre')[0].contents[0]), soup.find_all('pre')[1].contents[0]



def wyoming_pandas_to_dict(df):
    # extract metadata
    metadata = {k: v for d in df._metadata for k, v in d.items()}
    sounding_time = metadata['Observation time']
    date_sounding = datetime.datetime(int('20' + sounding_time[0:2]), int(sounding_time[2:4]), int(sounding_time[4:6]),

                                      int(sounding_time[7:9], int(sounding_time[9:11])))
    # build dictionary
    sounding = {}
    sounding['dimlabel'] = ['range']
    sounding['range'] = df['range'].values
    sounding['speed'] = (df['speed'].values * units.units('knots')).to_base_units().magnitude
    sounding['time'] = h.dt_to_ts(date_sounding)
    sounding['u_wind'] = (df['u_wind'].values * units.units('knots')).to_base_units().magnitude
    sounding['v_wind'] = (df['v_wind'].values * units.units('knots')).to_base_units().magnitude
    sounding['dewpoint'] = df['dewpoint']
    sounding['direction'] = df['direction']
    sounding['pressure'] = df['pressure']
    sounding['temperature'] = df['temperature']

    return sounding

def get_sounding(date, station_identifier):
    """Download Sounding from Uni Wyoming

        Args:
            date (datetime) of sounding of interest
            station_identifier (str), e.g. "SCCI" is Punta Arenas
        Returns:
            A dictionary containing the sounding data. More metadata (CAPE etc.) can be added later.

        """
    df = WyomingUpperAir.request_data(date, station_identifier)
    sounding = wyoming_pandas_to_dict(df)

    return sounding